//
//  BSViewController.m
//  MicroLifeDeviceSDK-Demo
//
//  Created by willy.wu on 2022/4/26.
//  Copyright © 2022 WiFi@MBP. All rights reserved.
//

#import "BSViewController.h"

@interface BSViewController ()

@end

@implementation BSViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    [self connectBloodSugar];
    [self.sdk device:@[self.bloodSugar]];
    kWeakSelf(self)
    [self.sdk getDidUpdateStateBlock:^(CBManagerState state) {
        NSString *log = [NSString stringWithFormat:@"DidUpdateState：%ld",(long)state];
        [weakself addLogWhitText:log];
    }];
    [self.sdk getScanDeviceBlock:^(id  _Nonnull device) {
        NSString *log = [NSString stringWithFormat:@"ScanDevice Name：%@ UUID：%@ mac：%@",[device name],[device UUID],[device mac]];
        [weakself addLogWhitText:log];
    } CancelScanBlock:^() {
        [weakself addLogWhitText:@"Cancel Scan"];
        [weakself.sdk startScan];
    }];
    [self.sdk getConnectDeviceStateBlock:^(id  _Nonnull device, CBCentralManager * _Nonnull central, CBPeripheral * _Nonnull peripheral, CBPeripheralState state, NSError * _Nonnull error) {
        [weakself connectDeviceInfo:device PeripheralState:state];
    } CancelAllDevicesConnectionBlock:^{
        [weakself addLogWhitText:@"Cancel All Devices Connection"];
        [weakself.sdk startScan];
    }];
}

- (void)viewDidLayoutSubviews {
    self.butView.contentSize = CGSizeMake(self.butView.frame.size.width, self.butView.frame.size.height*5);
}

#pragma mark - IBAction
- (IBAction)comAction:(UIButton *)sender {
    NSString *log = sender.titleLabel.text;
    [self addLogWhitText:log];
    switch (sender.tag) {
        case 100:
            [self.bloodSugar readDeviceName];
            break;
        case 101:
            [self.bloodSugar readFWVersion];
            break;
        case 102:
            [self.bloodSugar getBLEStatus];
            break;
        case 103:
            [self.bloodSugar readLastData];
            break;
        case 104:
            if (self.valueText.text.integerValue > 0) {
                [self.bloodSugar readData:self.valueText.text.integerValue];
            }
            break;
        case 105:
            [self.bloodSugar readDataLength];
            break;
        case 106:
            [self.bloodSugar disconnect];
            break;
        case 107:
            [self.bloodSugar readDateTime];
            break;
        case 108:
            [self.bloodSugar writeDeviceName:self.valueText.text];
            break;
        case 109:
            [self.bloodSugar getMeterDataTransmissionStatus];
            break;
        case 110:
            [self.bloodSugar getCommunicationMode];
            break;
        case 111:
            [self.bloodSugar setBLEParingMode];
            break;
        case 112:
            [self.bloodSugar setMeterAuthorization];
            break;
        case 113:
            [self.bloodSugar setParingRequest];
            break;
        case 114:
            [self.bloodSugar setParingTimeOut];
            break;
        default:
            break;
    }
}


- (void)connectBloodSugar {
    kWeakSelf(self)
    self.dataLength = 1;
    self.bloodSugar = [MicroLifeBloodSugar shareWhithAuthorizationkey:@""];
    [self.bloodSugar deviceConnectedBlock:^{
        NSString *log = [NSString stringWithFormat:@"%@ Connected",weakself.bloodSugar];
        [weakself addLogWhitText:log];
    }];
    [self.bloodSugar getReadDataModelBlock:^(NSInteger CMD, id  _Nonnull model, NSError * _Nonnull error) {
        [weakself log:weakself.bloodSugar CMD:CMD Model:model Error:error];
        switch (CMD) {
            case CMDBloodSugarReadDataLength:
                [weakself addValue:((MicroLifeBloodSugarMeasureData *)model).recordCounts.stringValue];
                break;
            case CMDBloodSugarReadDeviceName:
                [weakself addValue:((MicroLifeDeviceInfo *)model).BTModuleName];
            case CMDBloodSugarReadFWVersion:
            case CMDBloodSugarBLEStatus:
            case CMDBloodSugarReadData:
            case CMDBloodSugarReadNewData:
            case CMDBloodSugarReadHistorysData:
            case CMDBloodSugarReadLastData:
            case CMDBloodSugarDisconnect:
            case CMDBloodSugarReadDateTime:
                break;
            default:
                break;
        }
    }];
}

- (void)addValue:(NSString *)value {
    self.valueText.text = value;
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
