//
//  ECGViewController.m
//  MicroLifeDeviceSDK-Demo
//
//  Created by willy.wu on 2021/8/25.
//  Copyright © 2021 WiFi@MBP. All rights reserved.
//

#import "ECGViewController.h"

@interface ECGViewController () <UIScrollViewDelegate>
@end

@implementation ECGViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.ECGView.showsVerticalScrollIndicator = NO;
    self.ECGView.showsHorizontalScrollIndicator = NO;
    [self.ECGView addSubview:self.ecg];

    kWeakSelf(self)
    [self connectElectrocardiography];
    [self.sdk cancelScan];
    [self.sdk device:@[self.electrocardiography]];
    [self.sdk getDidUpdateStateBlock:^(CBManagerState state) {
        NSString *log = [NSString stringWithFormat:@"DidUpdateState：%ld",(long)state];
        [weakself addLogWhitText:log];
    }];
    [self.sdk getScanDeviceBlock:^(id  _Nonnull device) {
        NSString *log = [NSString stringWithFormat:@"ScanDevice Name：%@ UUID：%@ mac：%@",[device name],[device UUID],[device mac]];
        [weakself addLogWhitText:log];
    } CancelScanBlock:^() {
        [weakself addLogWhitText:@"Cancel Scan"];
        [weakself.sdk startScan];
    }];
    [self.sdk getConnectDeviceStateBlock:^(id  _Nonnull device, CBCentralManager * _Nonnull central, CBPeripheral * _Nonnull peripheral, CBPeripheralState state, NSError * _Nonnull error) {
        [weakself connectDeviceInfo:device PeripheralState:state];
        switch (state) {
            case CBPeripheralStateConnected:
                [weakself connectedWork];
                break;
            case CBPeripheralStateDisconnected:
                [weakself.sdk startScan];
                break;
            default:
                break;
        }
    } CancelAllDevicesConnectionBlock:^{
        [weakself addLogWhitText:@"Cancel All Devices Connection"];
    }];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
}

- (void)viewDidLayoutSubviews {
    self.scrollView.contentSize = CGSizeMake(self.scrollView.frame.size.width, self.scrollView.frame.size.height*2.5);
}

- (void)connectedWork {
    [self.dataArray removeAllObjects];
    self.BPM = @(0);
    self.ECGDatas = 0;
    self.LDOFF = YES;
    [self drawRealTimeECGLine];
}

#pragma mark -
- (NSMutableArray *)dataArray {
    if (!_dataArray) {
        self.dataArray = [NSMutableArray array];
    }
    return _dataArray;
}

- (NSArray *)low_p_fir_coeff {
    return @[@(0.0000000000000000000512756928033907),@(-0.0000134000069439525),@(-0.0000769592966179814),@(-0.000190506640751615),@(-0.000289464480572963),@(
             -0.000244839702462807),@(0.0000993780207259554),@(0.000847562763962751),@(0.00194831586362265),@(0.0031048157554041),@(
             0.00375616684339617),@(0.00318121648663095),@(0.000741916963369461),@(-0.00377519157968417),@(-0.00982357390477884),@(
             -0.0159099935555883),@(-0.0196876876188592),@(-0.0183899705678196),@(-0.00955002989961856),@(0.00816860613675884),@(
             0.0342863313261509),@(0.0662211513905311),@(0.0995625634895624),@(0.128901841120623),@(0.1490324596444),@(
             0.156198582897119),@(
             0.1490324596444),@(0.128901841120623),@(0.0995625634895624),@(0.0662211513905311),@(0.0342863313261509),@(
             0.00816860613675884),@(-0.00955002989961856),@(-0.0183899705678196),@(-0.0196876876188592),@(-0.0159099935555883),@(
             -0.00982357390477884),@(-0.00377519157968417),@(0.000741916963369461),@(0.00318121648663095),@(0.00375616684339617),@(
             0.0031048157554041),@(0.00194831586362265),@(0.000847562763962751),@(0.0000993780207259554),@(-0.000244839702462807),@(
             -0.000289464480572963),@(-0.000190506640751615),@(-0.0000769592966179814),@(-0.0000134000069439525),@(0.0000000000000000000512756928033907)];
}

- (CGRect)frameZero {
    CGFloat sEach = [UIScreen getPerMillimetreOfPT];
    NSInteger count = ceilf((SCREENWIDTH - 2*DDLeftPadding)/sEach);
    CGFloat width = count * sEach + 0.5;
    CGFloat height = 8 * sEach * 5 + 1;
    CGRect frame = CGRectMake(0, 0, width, height);
    return frame;
}

- (YOECGChartView *)ecg {
    if (!_ecg) {
        self.ecg = [[YOECGChartView alloc]initWithFrame:self.frameZero];
        self.ecg.standard.sampleFrequency = 256;
        self.ecg.standard.voltageUnit = YOECGChartViewVoltageUnitMilliVolt;
        [self.ecg reload];
    }
    return _ecg;
}

- (void)sampleFrequency:(int)sampleFrequency {
    if (self.dataArray.count == 0) {
        self.ecg.standard.sampleFrequency = sampleFrequency;
        [self.ecg reload];
    }
}

#pragma mark - device
- (void)connectElectrocardiography0 {
    kWeakSelf(self)
    MicroLifeElectrocardiography *ecgsdk = [MicroLifeElectrocardiography shareWhithAuthorizationkey:@""];
    [ecgsdk getDidUpdateStateBlock:^(CBManagerState state) {
        NSString *log = [NSString stringWithFormat:@"DidUpdateState：%ld",(long)state];
        [weakself addLogWhitText:log];
    } ScanDeviceBlock:^(id  _Nonnull device) {
        NSString *log = [NSString stringWithFormat:@"ScanDevice Name：%@ UUID：%@ mac：%@",[device name],[device UUID],[device mac]];
        [weakself addLogWhitText:log];
    } CancelScanBlock:^{
        [weakself addLogWhitText:@"Cancel Scan"];
    } ConnectDeviceStateBlock:^(id  _Nonnull device, CBCentralManager * _Nonnull central, CBPeripheral * _Nonnull peripheral, CBPeripheralState state, NSError * _Nonnull error) {
        [weakself connectDeviceInfo:device PeripheralState:state];
        switch (state) {
            case CBPeripheralStateConnected:
                [weakself connectedWork];
                break;
            default:
                break;
        }
    } CancelAllDevicesConnectionBlock:^{
        [weakself addLogWhitText:@"Cancel All Devices Connection"];
    }];
    [ecgsdk getReadDataModelBlock:^(NSInteger CMD, id  _Nonnull model, NSError * _Nonnull error) {
        [weakself showECG:model];
        [weakself showBPM:model];
    }];
}

- (void)connectElectrocardiography {
    kWeakSelf(self)
    self.electrocardiography = [MicroLifeECG shareWhithAuthorizationkey:@""];
    [self.electrocardiography deviceConnectedBlock:^{
        [weakself.electrocardiography writeDeviceTime];
        [weakself.electrocardiography readUserAndVersionData];
        [weakself.electrocardiography readDeviceStatus];
    }];
    [self.electrocardiography getReadDataValueBlock:^(NSInteger CMD, NSData * _Nonnull value, NSError * _Nonnull error) {
        switch (CMD) {
            case CMDECGReadECGRawData256hz:
                [weakself writeRowData:value];
                break;
            case CMDECGReadECGRawData512hz:
                [weakself writeRowData:value];
                break;
            default:
                break;
        }
    }];
    [self.electrocardiography getReadDataModelBlock:^(NSInteger CMD, id  _Nonnull model, NSError * _Nonnull error) {
        switch (CMD) {
            case CMDECGReadECGRawData256hz:
                [weakself sampleFrequency:256];
                [weakself showECG:model];
                [weakself showBPM:model];
                break;
            case CMDECGReadECGRawData512hz:
                [weakself sampleFrequency:512];
                [weakself showECG:model];
                [weakself showBPM:model];
                break;
            case CMDECGReadDataAndList:
                if (weakself.settingValue.SW_continued_ECG.boolValue) {
                    return;
                }
                [weakself readAllHistorys:model];
                [weakself log:weakself.electrocardiography CMD:CMD Model:model Error:error];
                break;
            case CMDECGReadDeviceUserSettingValues:
                if (weakself.settingValue.SW_continued_ECG.boolValue) {
                    return;
                }
                [weakself readSettingValue:model];
                [weakself log:weakself.electrocardiography CMD:CMD Model:model Error:error];
                break;
            case CMDECGReadDeviceCurrentStatus:
                if (weakself.settingValue.SW_continued_ECG.boolValue) {
                    return;
                }
                [weakself log:weakself.electrocardiography CMD:CMD Model:model Error:error];
                [weakself logDeviceCurrentStatus:model];
                break;
            default:
                if (weakself.settingValue.SW_continued_ECG.boolValue) {
                    return;
                }
                [weakself log:weakself.electrocardiography CMD:CMD Model:model Error:error];
                break;
        }
    }];
}

- (void)logDeviceCurrentStatus:(MicroLifeECGDeviceCurrentStatus *)deviceCurrentStatus {
    NSString *log = @"MicroLife ECG Status: ";
    switch (deviceCurrentStatus.DeviceStatus) {
        case MicroLifeECGStatusManualEnterBluetooth:
            log = [log stringByAppendingFormat:@"Manual Enter Bluetooth"];
            break;
        case MicroLifeECGStatusDetectNewBloodPressureData:
            log = [log stringByAppendingFormat:@"Detect New Blood Pressure Data"];
            break;
        case MicroLifeECGStatusDetectNewBloodPressureDataWaitECGTrigger:
            log = [log stringByAppendingFormat:@"Detect New Blood Pressure Data Wait ECG Trigger"];
            break;
        case MicroLifeECGStatusOnlyECGModeWaitECGTrigger:
            log = [log stringByAppendingFormat:@"Only ECG Mode Wait ECG Trigger"];
            break;
        case MicroLifeECGStatusWaitLeadOnStable:
            log = [log stringByAppendingFormat:@"Wait Lead On Stable"];
            break;
        case MicroLifeECGStatusStartECGMeasurement:
            log = [log stringByAppendingFormat:@"Start ECG Measurement"];
            break;
        case MicroLifeECGStatusStopECGMeasurement:
            log = [log stringByAppendingFormat:@"Stop ECG Measurement"];
            break;
        case MicroLifeECGStatusSucceedAndStoreMemory:
            log = [log stringByAppendingFormat:@"Succeed And Store Memory"];
            break;
        case MicroLifeECGStatusFactoryMode:
            log = [log stringByAppendingFormat:@"Factory Mode"];
            break;
        default:
            break;
    }
    log = [log stringByAppendingFormat:@" ,ECGcode:%ld",deviceCurrentStatus.ECGcode];
    [self addLogWhitText:log];
}

- (void)readAllHistorys:(nonnull MicroLifeECGDRecord *)data {
    if ([data isKindOfClass:[MicroLifeECGDRecord class]]) {
        self.historyMeasuremeNumber.text = data.historyMeasuremeNumber.stringValue;
    }
}

- (void)readSettingValue:(nonnull MicroLifeECGSettingValues *)settingValue {
    self.settingValue = settingValue;
    self.SWBT.on = self.settingValue.SW_BT.boolValue;
    self.SWBPplusECG.on = self.settingValue.SW_BPplusECG.boolValue;
    self.SWmax3003filter.on = self.settingValue.SW_max3003_filter.boolValue;
    self.SWovermemory.on = self.settingValue.SW_over_memory.boolValue;
    self.SWcontinuedECG.on = self.settingValue.SW_continued_ECG.boolValue;
    self.MAMInterval.text = [NSString stringWithFormat:@"%ld",self.settingValue.MAMInterval];
    self.length.text = [NSString stringWithFormat:@"%ld",self.settingValue.length];
}

#pragma mark - show
- (void)showECG:(MicroLifeECGRawData *)model {
    [self writeData:model.parseDictionary];
    NSMutableArray *arr = [NSMutableArray array];
    for (MicroLifeECGSampleData *data in model.ECGs) {
        NSNumber *row = [self filter:data.ECG];
        [arr addObject:row];
        [self.dataArray addObject:row];
    }
    [self.ecg drawRealTimeECGLine:arr];
}

- (void)showBPM:(MicroLifeECGRawData *)model {
    if (self.settingValue.SW_continued_ECG.boolValue) {
        return;
    }
    self.ECGDatas += model.ECGs.count;
    [self addLogWhitText:[NSString stringWithFormat:@"ECGDatas：%ld",self.ECGDatas]];
    if (self.BPM.integerValue != model.BPM.integerValue) {
        self.BPM = model.BPM;
        [self addLogWhitText:[NSString stringWithFormat:@"BPM：%@",self.BPM]];
    }
    if (self.LDOFF != model.LDOFF.boolValue) {
        self.LDOFF = model.LDOFF.boolValue;
        [self addLogWhitText:[NSString stringWithFormat:@"LDOFF：%@",self.LDOFF?@"EROOR":@"OK"]];
    }
}

#pragma mark - AVG
- (NSNumber *)filter:(NSNumber *)rawdata {
    float raw = 0.0;
    switch (self.showECGDataType) {
        case ECGDataTypeROW:
            raw = rawdata.floatValue*0.96;
            break;
        case ECGDataTypeLowPFirfilter:
            raw = [self low_p_firfilterECG:rawdata];
            break;
        case ECGDataTypeMovingAvgFilter:
            raw = [self moving_avg_filter:rawdata];
            break;
        case ECGDataTypeTwoAvgFilter:
            raw = [self two_avg_filter:rawdata];
            break;
        default:
            break;
    }
    return @(raw);
}

- (float)moving_avg_filter:(NSNumber *)rawdata {
    [self.dataArray insertObject:rawdata atIndex:0];
    NSInteger index = self.dataArray.count > 8 ? 8 : self.dataArray.count;
    float temp_avg = 0.0;
    for (int i = 0; i < index; i++) {
        temp_avg += [self.dataArray[i] floatValue];
    }
    temp_avg = (temp_avg/index);
    return temp_avg;
}

- (float)low_p_firfilterECG:(NSNumber *)rawdata {
    [self.dataArray insertObject:rawdata atIndex:0];
    float temp_fir = 0.0;
    NSInteger index = self.dataArray.count > 51 ? 51 : self.dataArray.count;
    for (int i = 0; i < index; i++) {
        temp_fir += [self.dataArray[i] floatValue]*[self.low_p_fir_coeff[i] floatValue];
    }
    return temp_fir;
}

- (float)two_avg_filter:(NSNumber *)rawdata {
    float temp_fir = [self low_p_firfilterECG:rawdata];
    temp_fir = [self moving_avg_filter:@(temp_fir)];
    return temp_fir;
}

#pragma mark - IBAction
- (IBAction)pdfAction:(id)sender {
    NSMutableArray *shares = [NSMutableArray array];
    if (self.ecg.pdfPath) {
        NSURL *pdfURL =[NSURL fileURLWithPath:self.ecg.pdfPath];
        [shares addObject:pdfURL];
    }
    for (UIImage *image in self.ecg.pdfImages) {
        [shares addObject:image];
    }
    if (shares.count > 0) {
        [self shareWithActivityItems:shares CallBlock:^(BOOL completed) {
            
        }];
    }
}

- (IBAction)comAction:(UIButton *)sender {
    switch (sender.tag) {
        case 100:
            [self drawStaticECGLine];
            break;
        case 101:
            [self drawRealTimeECGLine];
            break;
        case 102:
            [self drawStaticECGPDF];
            break;
        case 200:
            [self.electrocardiography readDeviceStatus];
            break;
        case 201:
            [self connectedWork];
            [self.electrocardiography startECGMeasurement:TransmitRate512Hz];
            break;
        case 202:
            [self.electrocardiography stopECGMeasurements];
            break;
        case 300:
            [self.electrocardiography readAllHistorys];
            break;
        case 301:
            [self connectedWork];
            [self.electrocardiography readECGDataWithIndex:self.historyMeasuremeNumber.text.integerValue TransmitRate:TransmitRate512Hz];
            break;
        case 302:
            [self.electrocardiography readECGInformationDataWithIndex:self.historyMeasuremeNumber.text.integerValue];
            break;
        case 303:
            [self.electrocardiography clearAllHistorys];
            break;
        case 304:
            [self.electrocardiography disconnect];
            break;
        case 305:
            [self.electrocardiography readUserAndVersionData];
            break;
        case 306:
        {
            NSInteger age = arc4random_uniform(99);
            [self.electrocardiography writeUserData:[self getBPMuserID] Age:age];
        }
            break;
        case 307:
            [self.electrocardiography readLastData];
            break;
        case 308:
            [self.electrocardiography clearLastData];
            break;
        case 309:
            [self.electrocardiography readDeviceInfo];
            break;
        case 310:
            [self.electrocardiography readDeviceTime];
            break;
        case 311:
            [self.electrocardiography writeDeviceTime];
            break;
        case 312:
            [self.electrocardiography readSerialNumber];
            break;
        case 313:
            [self.electrocardiography readFunctionSettingValue];
            break;
        case 314:
            [self.electrocardiography readSettingValues];
            break;
        case 315:
            if (self.settingValue) {
                [self.electrocardiography writeSettingValuesWithSW_BT:self.settingValue.SW_BT.boolValue SW_BPplusECG:self.settingValue.SW_BPplusECG.boolValue SW_max3003_filter:self.settingValue.SW_max3003_filter.boolValue SW_over_memory:self.settingValue.SW_over_memory.boolValue SW_continued_ECG:self.settingValue.SW_continued_ECG.boolValue MAMInterval:self.settingValue.MAMInterval length:self.settingValue.length];
            }  else {
                [self addLogWhitText:@"Please, readSettingValues ​​first"];
            }
            break;
        case 316:
            [self.electrocardiography askDeviceDisplayTransmitEndAnimation];
            break;
        default:
            break;
    }
}

- (IBAction)firfilterAction:(id)sender {
    [self.electrocardiography disconnect];
    [self.sdk startScan];
    NSArray *dataSource = @[@"ROW",@"low_p_firfilter",@"moving_avg_filter",@"two_avg_filter"];
    NSString *selectedItem = dataSource[self.showECGDataType];
    kWeakSelf(self)
    IdeabusPickView *ideabusPickView = [[IdeabusPickView alloc]initWithDataSource:dataSource selectedItem:selectedItem titleName:@"ShowECGDataType" confirmButtonName:@"Save" confirmBlock:^(id item) {
        [weakself firfilterTitle:item];
        weakself.showECGDataType = [dataSource indexOfObject:item];
    } cancelButtonName:NSLocalizedString(@"Cancel",nil) cancelBlock:^(id item) {
        
    } hideBlock:^{
        
    }];
    [ideabusPickView show];
}

- (void)firfilterTitle:(NSString *)item {
    kWeakSelf(self)
    dispatch_async(dispatch_get_main_queue(), ^{
        [weakself.Firfilter setTitle:item forState:UIControlStateNormal];
    });
}

- (void)drawStaticECGLine {
    float index = self.ecg.standard.oneGridSize*2;
    float width = (self.dataArray.count * self.ecg.standard.onePointWidth) + index;
    float height = self.ecg.frame.size.height + index;
    self.ECGView.contentSize = CGSizeMake(width, height);
    self.ECGView.scrollEnabled = YES;
    [self.ecg reload];
    [self.ecg drawStaticECGLine:self.dataArray];
}

-(void)drawRealTimeECGLine {
    [self.dataArray removeAllObjects];
    [self.ECGView setContentOffset:CGPointZero animated:NO];
    self.ECGView.scrollEnabled = NO;
    self.ecg.frame = self.frameZero;
    [self.ecg reload];
    [self.ecg drawRealTimeECGLine:self.dataArray];
}

- (void)drawStaticECGPDF {
    float allWidth = self.dataArray.count * self.ecg.standard.onePointWidth;
    float ecgWidth = 5 * 10 * self.ecg.standard.oneGridSize;
    int indexH = ceil(allWidth/ecgWidth);
    float contentWidth = ecgWidth + self.ecg.standard.oneGridSize*2;
    float contentHeight = (self.ecg.frame.size.height * indexH) + (self.ecg.standard.oneGridSize * 3 * indexH);
    self.ECGView.contentSize = CGSizeMake(contentWidth, contentHeight);
    self.ECGView.scrollEnabled = YES;
//    self.ecg.statictiveMaxNum = 30;
    [self.ecg reload];
    [self.ecg drawStaticECGLine:self.dataArray oneLine:NO];
}

- (IBAction)switchSettingValues:(UISwitch *)sender {
    NSString *log = [NSString stringWithFormat:@"Switch SettingValues:"];
    switch ([sender tag]) {
        case 100:
            log = [log stringByAppendingFormat:@"SW_BT"];
            self.settingValue.SW_BT = @(sender.on);
            break;
        case 101:
            log = [log stringByAppendingFormat:@"SW_BPplusECG"];
            self.settingValue.SW_BPplusECG = @(sender.on);
            break;
        case 102:
            log = [log stringByAppendingFormat:@"SW_max3003_filter"];
            self.settingValue.SW_max3003_filter = @(sender.on);
            break;
        case 103:
            log = [log stringByAppendingFormat:@"SW_over_memory"];
            self.settingValue.SW_over_memory = @(sender.on);
            break;
        case 104:
            log = [log stringByAppendingFormat:@"SW_continued_ECG"];
            self.settingValue.SW_continued_ECG = @(sender.on);
            break;
        default:
            break;
    }
    log = [log stringByAppendingFormat:@" %@",sender.on?@"YES":@"NO"];
    [self addLogWhitText:log];
}

- (void)editSettingValues:(UITextField *)sender {
    NSString *log = [NSString stringWithFormat:@"Edit SettingValues:"];
    switch ([sender tag]) {
        case 105:
            log = [log stringByAppendingFormat:@"MAMInterval: %@",sender.text];
            self.settingValue.MAMInterval = sender.text.integerValue;
            break;
        case 106:
            log = [log stringByAppendingFormat:@"length %@",sender.text];
            self.settingValue.length = sender.text.integerValue;
            break;
        default:
            break;
    }
    [self addLogWhitText:log];
}

- (IBAction)editMAMInterval:(UITextField *)sender {
    [self.view endEditing:YES];
    NSMutableArray *dataSource = [NSMutableArray arrayWithObjects:@"15", @"30", @"60" ,@"120", nil];
    IdeabusPickView *ideabusPickView = [[IdeabusPickView alloc] initWithDataSource:dataSource selectedItem:sender.text titleName:@"Interval" confirmButtonName:NSLocalizedString(@"Save",nil) confirmBlock:^(id item) {
        sender.text = item;
        [self editSettingValues:sender];
    } cancelButtonName:NSLocalizedString(@"Cancel",nil) cancelBlock:^(id item) {
        sender.text = item;
        [self editSettingValues:sender];
    } hideBlock:^{
        
    }];
    [ideabusPickView show];
}

- (IBAction)editLength:(UITextField *)sender {
    [self.view endEditing:YES];
    NSMutableArray *dataSource = [NSMutableArray arrayWithObjects:@"0",@"1",@"2", nil];
    IdeabusPickView *ideabusPickView = [[IdeabusPickView alloc] initWithDataSource:dataSource selectedItem:sender.text titleName:@"Length" confirmButtonName:NSLocalizedString(@"Save",nil) confirmBlock:^(id item) {
        sender.text = item;
        [self editSettingValues:sender];
    } cancelButtonName:NSLocalizedString(@"Cancel",nil) cancelBlock:^(id item) {
        sender.text = item;
        [self editSettingValues:sender];
    } hideBlock:^{
        
    }];
    [ideabusPickView show];
}
/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
