//
//  MicroLifeECG.h
//  MicroLifeDeviceSDK
//
//  Created by willy.wu on 2022/7/19.
//

#import <MicroLifeDeviceSDK/MicroLifeDeviceSDK.h>

NS_ASSUME_NONNULL_BEGIN

typedef NS_ENUM(NSInteger , CMDECG) {
    CMDECGReadECGRawData256hz = 0x69,
    CMDECGReadECGRawData512hz = 0x70,
    CMDECGReadDataAndList = 0x00,
    CMDECGReadECGDataByIndex = 0x0100,
    CMDECGReadECGInformationDataByIndex = 0x0101,
    CMDECGClearTheAllHistoryData = 0x03,
    CMDECGDisconnect = 0x04,
    CMDECGReadUserIDAndVersionData = 0x05,
    CMDECGWriteNewUserID = 0x06,
    CMDECGReadLastData = 0x07,
    CMDECGClearLastData = 0x08,
    CMDECGReadDeviceIDAndInfo = 0x0B,
    CMDECGReadDeviceTime = 0x0C,
    CMDECGWriteDeviceTime = 0x0D,
    CMDECGAskDeviceDisplayTransmitAnimationEnd = 0x0E,
    CMDECGReadSerialNumber = 0x0F,
    CMDECGReadDeviceFunctionSettingValues = 0x10,
    CMDECGReadDeviceUserSettingValues = 0x14,
    CMDECGwriteDeviceUserSettingValues = 0x15,
    CMDECGReadDeviceCurrentStatus = 0x20,
    CMDECGStartECGMeasurement = 0x25,
    CMDECGStopECGMeasurement = 0x26,
    CMDECGWriteSerialNumber = 0x0F01,
    CMDECGWriteDeviceFunctionSettingValues = 0x11,
    CMDECGReadECGAllSettingValues = 0x12,
    CMDECGWriteECGAllSettingValues = 0x13
};

@interface MicroLifeECG : MicroLifeDevice

/// Read data and list from device
- (void)readAllHistorys;

/// Send ECG raw data by index to APP
/// @param index Valid memory index rule:Check by command 00h which reply condition = “BP_ECG (31)” or “ECG (32)” and ECG code = “1: succeed measurement”.
/// @param transmitRate Transmit Rate:Valid range: 1~2 1:256Hz 2:512Hz
- (void)readECGDataWithIndex:(NSInteger)index TransmitRate:(TransmitRate)transmitRate;

/// Send ECG information data by index to APP
/// @param index Valid memory index rule:Check by command 00h which reply condition = “BP_ECG (31)” or “ECG (32)” and ECG code = “1: succeed measurement”.
- (void)readECGInformationDataWithIndex:(NSInteger)index;

/// Clear all history data of the DEVICE
- (void)clearAllHistorys;

/// Disconnect the Bluetooth
- (void)disconnect;

/// Read user ID and version data from DEVICE
- (void)readUserAndVersionData;

/// Write a new user ID from DEVICE
/// @param ID user ID
/// @param age age
- (void)writeUserData:(NSString *)ID Age:(NSInteger)age;

/// Read Last 1 data from device
/// ※This command is only used while device-status is “command 20h-2 or 20h-3 (detect new blood pressure data).
- (void)readLastData;

/// Clear Last 1 data of device
/// ※This command is only used while device-status is “command 20h-2 or 20h-3 (detect new blood pressure data).
- (void)clearLastData;

/// Read device ID and info from DEVICE
- (void)readDeviceInfo;

///  Read device Time from BPM
- (void)readDeviceTime;

/// Write device Time to BPM
- (void)writeDeviceTime;

/// ask device display transmit-end animation
/// If any command succeeds, LCD displays  icon animation (flash at 1Hz).
/// Send this command to display transmit-end status ( flash 3 times)
- (void)askDeviceDisplayTransmitEndAnimation;

/// Read serial number from DEVICE
- (void)readSerialNumber;

/// Read device function setting values from device
- (void)readFunctionSettingValue;

/// Read device user setting values from device
- (void)readSettingValues;

/// Write device user setting values from device
/// @param SW_BT Turn of all Bluetooth user interface like airplane mode. If the use set this bit by Bluetooth, device is effective after disconnection. User can turn on this function by device setting mode. (refer to user manual)
/// @param SW_BPplusECG 0: only measure blood pressure 1: After blood pressure measurement, device will auto start ECG trigger by arrthymia.
/// @param SW_max3003_filter 0:turn off frontend filter 1:turn on frontend filter
/// @param SW_over_memory 0: If the memory records are equal to 40, device will display “ful” and not stop measure until clear memory. 1: If the memory is full, device will clear the oldest memory to store new data.
/// @param SW_continued_ECG 0: The measure time of ECG is 30 or 60 seconds, and raw data will be stored to flash memory. 1: The measure time of ECG is unlimited for test purpose. The raw data won’t be stored to flash memory.
/// @param MAMInterval 15, 30, 60,120
/// @param length 0:N/A, 1:30 seconds, 2:60seconds
- (void)writeSettingValuesWithSW_BT:(BOOL)SW_BT SW_BPplusECG:(BOOL)SW_BPplusECG SW_max3003_filter:(BOOL)SW_max3003_filter SW_over_memory :(BOOL)SW_over_memory  SW_continued_ECG:(BOOL)SW_continued_ECG  MAMInterval:(MAMInterval)MAMInterval length:(SsampleTime)length;

/// Read Device Status
- (void)readDeviceStatus;

/// Start ECG measurement
/// @param transmitRate 01h: 256Hz 02h: 512Hz
/// After this command, device will enter On-line mode(during ECG measurement) and continued update lead-status every 1 seconds. After lead-on stable, device will update ready to start status.
- (void)startECGMeasurement:(TransmitRate)transmitRate;

/// Stop ECG measurements
- (void)stopECGMeasurements;
@end

NS_ASSUME_NONNULL_END
